import 'package:dio/dio.dart';

import 'logger.dart';

class PrettyDioLogger extends Interceptor {
  /// Print request [Options]
  final bool request;

  /// Print request header [Options.headers]
  final bool requestHeader;

  /// Print request data [Options.data]
  final bool requestBody;

  /// Print [Response.data]
  final bool responseBody;

  /// Print [Response.headers]
  final bool responseHeader;

  /// Print error message
  final bool error;

  /// Log printer; defaults print log to console.
  /// In flutter, you'd better use debugPrint.
  /// you can also write log in a file, for example:
  ///```dart
  ///  var file=File("./log.txt");
  ///  var sink=file.openWrite();
  ///  dio.interceptors.add(LogInterceptor(logPrint: sink.writeln));
  ///  ...
  ///  await sink.close();
  ///```
  void Function(Object object) logPrint;

  PrettyDioLogger({
    this.request = false,
    this.requestHeader = false,
    this.requestBody = true,
    this.responseHeader = false,
    this.responseBody = true,
    this.error = true,
    this.logPrint = print,
  });

  @override
  void onRequest(
      RequestOptions options, RequestInterceptorHandler handler) async {
    StringBuffer sb = StringBuffer(
        '🌏 REQUEST -> [${options.method}] with URL: ${options.uri}\n');
    if (request) {
      sb.writeln(_line);
      sb.writeln(_pairKV('Method', options.method));
      sb.writeln(_pairKV('ResponseType', options.responseType.toString()));
      sb.writeln(_pairKV('FollowRedirects', options.followRedirects));
      sb.writeln(_pairKV('ConnectTimeout', options.connectTimeout));
      sb.writeln(_pairKV('SendTimeout', options.sendTimeout));
      sb.writeln(_pairKV('ReceiveTimeout', options.receiveTimeout));
      sb.writeln(_pairKV(
          'ReceiveDataWhenStatusError', options.receiveDataWhenStatusError));
      sb.writeln(_pairKV('Extra', options.extra));
    }
    if (requestHeader) {
      sb.writeln(_line);
      sb.writeln('Headers: ');
      options.headers.forEach((key, value) => sb.writeln(_pairKV(key, value)));
    }
    if (requestBody) {
      sb.writeln(_line);
      sb.writeln('Data: ');
      options.data.toString().split('\n').forEach((data) => sb.writeln(data));
    }
    logger.i(sb.toString());
    handler.next(options);
  }

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) async {
    StringBuffer sb =
        StringBuffer('✈️ RESPONSE -> URL: ${response.requestOptions.uri}\n');
    _printResponse(sb, response);
    handler.next(response);
  }

  @override
  void onError(DioError err, ErrorInterceptorHandler handler) async {
    if (error) {
      StringBuffer sb =
          StringBuffer('❌❌❌ DIO ERROR -> URL: ${err.requestOptions.uri}\n');
      if (err.response != null) {
        _printResponse(sb, err.response!);
      }
    }
    handler.next(err);
  }

  void _printResponse(StringBuffer sb, Response response) {
    int statusCode = response.statusCode ?? 0;
    bool isSuccess = statusCode >= 200 && statusCode < 300;
    String presentStatus = isSuccess ? '✅' : '❌';
    sb.writeln('StatusCode: $statusCode $presentStatus');
    if (response.isRedirect == true) {
      sb.writeln(_pairKV('Redirect', response.realUri));
    }
    if (responseHeader) {
      sb.writeln(_line);
      sb.writeln('Headers: ');
      response.headers.forEach((key, value) => sb.writeln(_pairKV(key, value)));
    }
    if (responseBody) {
      sb.writeln(_line);
      sb.writeln('Response Text: ');
      response.toString().split('\n').forEach((data) => sb.writeln(data));
    }
    if (isSuccess) {
      logger.i(sb.toString());
    } else {
      logger.e(sb.toString());
    }
  }

  String _pairKV(String key, Object? value) => '$key: $value';

  String get _line => '---------------------------------------------';
}
