import 'dart:math' as math;

import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:flutter_staggered_grid_view/flutter_staggered_grid_view.dart';
import 'package:knt_bloc/knt_bloc.dart';

import '../../bloc/paging_gallery_bloc.dart';
import '../../defined.dart';
import '../../model/models.dart';
import '../image/knt_image.dart';
import '../knt_ripple.dart';
import 'knt_gallery_page.dart';
import 'knt_photo_browser_page.dart';

const _defaultStyleCount = 2;

/// Staggered Gallery with default 2 styles
///
/// Photos will be separated into blocks, each block has [numberPhotoInBlock] photos
class KntStaggeredGallery extends StatelessWidget {
  const KntStaggeredGallery({
    Key? key,
    required this.photoList,
    this.maxPresentPhoto = DefinedUI.defaultMaxPresentCountGallery,
    int? numberPhotoInBlock,
    Axis? direction,
    this.mainSpacing,
    this.crossSpacing,
    this.padding,
    this.onPickedPhoto,
    this.onSeeAll,
    this.convertedSeeAllText,
    this.photoBorderRadius,
  })  : this.direction = direction ?? Axis.vertical,
        this.numberPhotoInBlock = numberPhotoInBlock ??
            (direction == Axis.horizontal
                ? DefinedUI.defaultPhotoCountInGalleryHorzBlock
                : DefinedUI.defaultPhotoCountInGalleryVertBlock),
        super(key: key);

  final List<Photo> photoList;
  final int maxPresentPhoto;
  final int numberPhotoInBlock;
  final Axis direction;
  final double? mainSpacing;
  final double? crossSpacing;
  final EdgeInsets? padding;
  final Function(int)? onPickedPhoto;
  final VoidCallback? onSeeAll;
  final double? photoBorderRadius;

  /// Convert number of hidden photos into text
  final String Function(int)? convertedSeeAllText;

  @override
  Widget build(BuildContext context) {
    final isVerticalScroll = direction == Axis.vertical;
    final numberHiddenPhoto = photoList.length - maxPresentPhoto;
    return StaggeredGridView.countBuilder(
      shrinkWrap: true,
      scrollDirection: direction,
      crossAxisCount: numberPhotoInBlock - 1,
      physics: isVerticalScroll
          ? const NeverScrollableScrollPhysics()
          : const BouncingScrollPhysics(),
      padding: padding ?? const EdgeInsets.all(0),
      itemCount: math.min(photoList.length, maxPresentPhoto),
      itemBuilder: (BuildContext context, int index) {
        final photo = photoList[index];
        final bigChunk = index % numberPhotoInBlock == 0 ||
            index % numberPhotoInBlock ==
                (numberPhotoInBlock - _defaultStyleCount);
        final hiddenPhotoBackground = SizedBox(
          child: (convertedSeeAllText != null &&
                  numberHiddenPhoto > 1 &&
                  index == maxPresentPhoto - 2)
              ? KntRipple(
                  onTap: () => _onSeeAll(context),
                  child: Container(
                    decoration: BoxDecoration(
                      color: Colors.black54,
                      borderRadius: BorderRadius.circular(16.w),
                    ),
                    alignment: Alignment.center,
                    child: Text(
                      convertedSeeAllText?.call(numberHiddenPhoto) ?? '',
                      textAlign: TextAlign.center,
                      style: TextStyle(
                        color: Colors.white,
                        fontSize: 18.sp,
                        fontWeight: FontWeight.w600,
                        height: 21.sp / 18.sp,
                      ),
                    ),
                  ),
                )
              : const SizedBox(),
        );
        return InkWell(
          onTap: () => _onPickedPhoto(context, index),
          child: Hero(
            tag: photo.id,
            child: KntImage(
              uri: bigChunk ? photo.coverUri : photo.mediumSizeUri,
              borderRadius: BorderRadius.circular(photoBorderRadius ?? 16.w),
              child: hiddenPhotoBackground,
              imageType: photo.storageType,
            ),
          ),
        );
      },
      staggeredTileBuilder: (int index) {
        if (isVerticalScroll) {
          // Each bloc has [numberPhotoInBlock] photos which shared into [_defaultStyleCount] styles
          final bigChunk = index % numberPhotoInBlock == 0 ||
              index % numberPhotoInBlock ==
                  (numberPhotoInBlock - _defaultStyleCount);
          return StaggeredTile.count(bigChunk ? 3 : 2, bigChunk ? 4 : 2);
        }
        final bigChunk = index % numberPhotoInBlock == 0;
        return StaggeredTile.count(bigChunk ? 4 : 1, bigChunk ? 1.6 : 1);
      },
      mainAxisSpacing: mainSpacing ?? 12.w,
      crossAxisSpacing: crossSpacing ?? 12.w,
    );
  }

  void _onPickedPhoto(BuildContext context, int index) {
    if (onPickedPhoto != null) {
      onPickedPhoto!.call(index);
    } else {
      KntGalleryPage.push(
        context,
        photoList: photoList,
        initIndex: index,
      );
    }
  }

  void _onSeeAll(BuildContext context) {
    onSeeAll?.call();
    context.read<PagingGalleryBloc>().add(LoadGalleryPhotosEvent(photoList));
    KntPhotoBrowserPage.push(context);
  }
}
